/* Copyright 1994, Roger Smith for Sterling Software and NASA-Ames Research Center
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of NASA and Sterling Software (collectively 
 * or individually) are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  NASA and Sterling Software makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * NASA & STERLING SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL NASA & STERLING SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * 1994 Revision
 * Author:           Roger Smith, Sterling Software @ NASA-Ames Research Center
 *                   Moffett Field, California, rsmith@proteus.arc.nasa.gov
 */
#include <sys/time.h>

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif
#ifndef NULL
#define NULL 0
#endif
#define MSG_INTERVAL 9
#define LONG_TIME 100000
#ifndef Bool
typedef ushort Bool;
#endif
	
struct _stats {
    double     minval;
    double     maxval;
    double     sum;
    double     mean;
    double     xsum;
    double     stddev;
    double     var;
    double     ccount;
};

/*
 * client side rpc statistics
 */
struct _rpc_c{
	int	rccalls;	        /* # client calls */
	int	rcbadcalls;		/* # client badcalls */
	int	rcretrans;		/* # client retrans */
	int	rcbadxids;		/* # client badxid's */
	int	rctimeouts;		/* # client timeouts */
	int	rcwaits;		/* # client waits */
	int	rcnewcreds;		/* # client newcreds */
	int	rcbadverfs;
	int	rctimers;
} ;

/*
 * client side nfs statistics
 */
struct _nfs_c{
	u_int	nclsleeps;		/* client handle waits */
	u_int	nclgets;		/* client handle gets */
	u_int	ncalls;			/* client requests */
	u_int	nbadcalls;		/* rpc failures */
	u_int	reqs[32];		/* count each request */
} ;

/*
 * Server side rpc statistics
 */
struct _rpc_s{
	int	rscalls;
	int	rsbadcalls;
	int	rsnullrecv;
	int	rsbadlen;
	int	rsxdrcall;
} ;

/*
 * server side nfs statistics
 */
struct _nfs_s{
	u_int	ncalls;		/* number of calls received */
	u_int	nbadcalls;	/* calls that failed */
	u_int	reqs[32];	/* count each request */
} ;

struct _sysStats {
    time_t           time_stamp;
    unsigned         mem_avm, mem_fre;
    unsigned         si, so, psi, pso;
    unsigned         re, at, pi, po, ppi, ppo, fr, de, sr;
    unsigned         pfr, tpf;
    unsigned         allo, res, tot;
    unsigned         ipkts, epkts, opkts, coll;
    unsigned         dkxfer;
    unsigned         intr;
    int              spare_i[5];
};
struct _procStats {
    time_t           time_stamp;
    unsigned         proc_r, proc_b, proc_w;
    float            cpu[4];
    float            one, five, fifteen;
    float            wakeupTime;
    float            forkTime;
    float            pipeTime;
    int              spare_i[2];
};
struct _rpcNnfs {
    time_t           time_stamp;
    struct _rpc_c    rpc_c;
    struct _rpc_s    rpc_s;
    struct _nfs_c    nfs_c;
    struct _nfs_s    nfs_s;
    int              spare_i[11];	/* to make size of structure fall on modulo128 boundry */
};

/***************************************************
 *
 *
struct _mydata {
    struct     timeval timeStamp;
    double     wakeupTime;
    int        userCpu;
    int        sysCpu;
    int        idleCpu;
    int        freeMem;
    int        diskXfr;
    int        intrpts;
    int        inpPkts;
    int        outPkts;
    int        colPkts;
    double     loadAvg;
    double     rtInetTime;
    double     forkTime;
    double     pipeTime;
    double     spare1;
    double     spare2[4];
};

#define MEM_B  00001
#define PAG_B  00002
#define SWP_B  00004
#define NET_B  00010
#define RPC_B  00020
#define NFS_B  00040
#define CLNT_B 00100
#define SRVR_B 00200
#define PERF_B 00400
#define PROC_B 01000

struct mainfile {
    short            filetype;
    time_t           time_stamp;
    unsigned         proc_r, proc_b, proc_w;
    unsigned         mem_avm, mem_fre;
    unsigned         si, so, psi, pso;
    unsigned         re, at, pi, po, ppi, ppo, fr, de, sr;
    unsigned         pfr, tpf;
    float            cpu[4];
    float            one, five, fifteen;
    unsigned         allo, res, tot;
    unsigned         ipkts, epkts, opkts, coll;
    unsigned         dkxfer;
    unsigned         intr;
    float            wakeupTime;
    float            forkTime;
    float            pipeTime;
    struct _rpc_c    rpc_c;
    struct _rpc_s    rpc_s;
    struct _nfs_c    nfs_c;
    struct _nfs_s    nfs_s;
    unsigned         spare_u;
    float            spare_f[2];
};
********************************************************/
