/*
 * $XConsortium: system.h,v 1.4 89/09/19 14:43:58 jim Exp $
 *
 * Perfmon Performance Monitor
 *
 * Copyright 1985, Massachusetts Institute of Technology
 * Copyright 1989 PCS Computer Systeme GmbH, West Germany
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of PCS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  PCS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * PCS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL PCS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Original Author:  Emanuel Jay Berkenbilt, MIT Project Athena
 * Author:           Thomas A. Baghli, PCS Computer Systeme GmbH, West Germany
 *                   tom@meepmeep.pcs.com
 * Copyright 1994, Roger Smith for Sterling Software and NASA-Ames Research Center
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of NASA and Sterling Software (collectively 
 * or individually) are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  NASA and Sterling Software makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * NASA & STERLING SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL NASA & STERLING SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * 1994 Revision
 * Author:           Roger Smith, Sterling Software @ NASA-Ames Research Center
 *                   Moffett Field, California, rsmith@proteus.arc.nasa.gov
 */



/* This file information needed by both system.c and Perfmon.c */

#define NUM_POSSIBLE_STATS     11
#define MAX_STATS NUM_POSSIBLE_STATS + 1

#define LINES_PER_LABEL 2

#ifdef PCS
#define CPUSTATES	4
#define DK_NDRIVE	4
#define NBPG		4096

#define USER		0
#define NICE		1
#define SYS		2
#define IDLE		3
#endif PCS

#define FORALLPOSSIBLESTATS(stat) \
    for (stat = 0; stat < NUM_POSSIBLE_STATS; stat++)
#define FORALLSTATS(stat) \
    for (stat = 0; stat < num_stats; stat++)

#define	NO_STAT			-1
#define USER_CPU_PERCENTAGE	0
#define SYSTEM_CPU_PERCENTAGE	1
#define IDLE_CPU_PERCENTAGE	2
#define FREE_MEM		3
#define DISK_TRANSFERS		4
#define INTERRUPTS		5
#define INPUT_PACKETS		6
#define OUTPUT_PACKETS		7
#define COLLISION_PACKETS	8
#define NFS_CLIENT_CALLS        9
#define NFS_SERVER_CALLS       10

struct statistic {
    int min_val, max_val;
    int value;
    char *label[LINES_PER_LABEL];
};

typedef struct statistic stat_type[MAX_STATS];


/* For setting and adjusting the timer. */

#define HUNDRED 100

/* These are passed to adjust_timeout() and represent
 * numbers of seconds.
 */
#define LARGE_ADD_TIME HUNDRED/10
#define LARGE_SUBTRACT_TIME -LARGE_ADD_TIME
#define SMALL_ADD_TIME LARGE_ADD_TIME/10
#define SMALL_SUBTRACT_TIME -SMALL_ADD_TIME
#define MAX_TIME 3 * HUNDRED /* five minutes */

#define VALID_UPDATE(x) ((x <= MAX_TIME) && (x > 0))

#define USR_CPU 0
#define SYS_CPU 1
#define IDL_CPU 2
#define FRE_MEM 3
#define DSK_XFR 4
#define INTRPTS 5
#define INP_PKT 6
#define OUT_PKT 7
#define COL_PKT 8
#define NFS_CLT 9
#define NFS_SRV 10
#define LAST_ITEM NFS_SRV	/* must equal last item above */
#define NUM_GRAPHS LAST_ITEM+1

/* data structure for custom application resources */
typedef struct {
    XtAppContext app_context;
    Widget toplevel;
    int interval;		/* update interval */
    int count;			/* nothing (yet) */
    Boolean fast;		/* when set, interval is in milliseconds */
    int ms_per_sec;		/* interval multiplier, set to 1 if fast is true */
    Boolean fill;		/* draw filled graph when set, else draw line graph */
    Boolean debug;		/* control debug output */
    Bool addG[NUM_GRAPHS];
    Bool subG[NUM_GRAPHS];
    Bool graphOn[NUM_GRAPHS];
    Dimension numGraphsOn;	/* number of graphs on and running for this invocation */
    XtIntervalId interval_id;	/* timeout interval id */
    Bool stuf;
}AppData, *AppDataPtr;

static char *topNames[] = {
    "User",
    "System",
    "Idle",
    "Free",
    "Disk",
    "Interrupts",
    "Input",
    "Output",
    "Collision",
    "NFS Client",
    "NFS Server",
};
static char *widgetLabels[] = {
    "User",
    "System",
    "Idle",
    "Free",
    "Disk",
    "Intrpts",
    "Input",
    "Output",
    "Collision",
    "NFSClient",
    "NFSServer",
};
static char *botNames[] = {
    "CPU",
    "CPU",
    "CPU",
    "Memory",
    "Transfers",
    "",
    "Packets",
    "Packets",
    "Packets",
    "Calls",
    "Calls",
};
